namespace Helmut
{
int numMovesToSolve(const Level& level)
{
	//Wir berechnen jede Kombination von Spielzuständen und speichern in einem Bitarray,
	//welcher Zustand erreicht werden kann. Wir gehen dabei gleichzeitig von der Anfangs-
	//position und der Endposition aus, bis sich ein Zustand überlappt. Die Zustände
	//rückwärts zu durchwandern klingt im ersten Moment schwierig, ist aber eigentlich
	//ganz einfach. Man kann sich einfach vorstellen, dass sich im Level Spielfigur und
	//Zielfeld die Plätze tauschen. Außerdem ändern sich die Regeln ein wenig. Jede Figur
	//kann nur in eine Richtung rutschen, wenn sie sich mit dem Rücken zur Wand abstoßen
	//kann (oder einer anderen Figur). Dafür kann sie jederzeit bremsen (tut das aber
	//sinnvollerweise nur dann, wenn sie Kontakt zu einer Wand hat, sonst kann sie nicht
	//weiter).
	//Das Rückwärts-Durchwandern erzeugt natürlich etwas mehr Zustände als vorwärts, etwa
	//Faktor 5, aber da die Zahl der Zustände pro Schritt etwa exponentiell wächst und wir
	//dadurch im Durchschnitt die Hälfte der Schritte sparen lohnt sich das trotzdem.
	//Sogar ziemlich deutlich. Ich habe mal das mal geplottet: http://bit.ly/YON7bO
	//Der blaue Graph zeigt, wie viele Spielzustände man im ersten Beispiellevel mit
	//x (und nicht weniger als x) Zügen erreichen kann. Der rote Graph zeigt die Anzahl
	//der möglichen Spielzustände, in denen man das Spiel in genau 14-x Zügen gewinnen
	//kann.
	//Da die Laufzeit der Funktion proportional zur Zahl der berechneten Zustände und
	//zur Zahl der Schritte ist entspricht das etwa der Fläche unter den Graphen.
	//Mein Algo rechnet immer an der Seite den nächsten Schritt, wo es die wenigsten
	//Zustände gibt, entspricht also dem Minimum der beiden Funktionen. Wenn man nur
	//vorwärts rechnet entspricht das der Fläche unter dem blauen Graphen.


	struct border
	{
		int left, right, top, bottom;
	};
	border borders[16][16];//In jede Richtung speichern wo die nächste Mauer ist.
	for(int y = 1; y < 15; y++)
	{
		int b = 1;
		for(int x = 1; x < 15; x++)
		{
			if(level[x][y] == '#')
				b = x+1;
			borders[x][y].left = b;
		}
		b = 14;
		for(int x = 14; x >= 1; x--)
		{
			if(level[x][y] == '#')
				b = x-1;
			borders[x][y].right = b;
		}
	}
	for(int x = 1; x < 15; x++)
	{
		int b = 1;
		for(int y = 1; y < 15; y++)
		{
			if(level[x][y] == '#')
				b = y+1;
			borders[x][y].top = b;
		}
		b = 14;
		for(int y = 14; y >= 1; y--)
		{
			if(level[x][y] == '#')
				b = y-1;
			borders[x][y].bottom = b;
		}
	}
	int state = 0;//Anfangszustand, eine Position pro Byte
	int stateR = 0;//Endzustand
	for(int x = 1; x < 15; x++)
	for(int y = 1; y < 15; y++)
	{
		switch(level[x][y])
		{
		case 'r': state  |= (x << 0 ) | (y << 4 ); break;
		case 'g': state  |= (x << 8 ) | (y << 12); break;
		case 'b': state  |= (x << 16) | (y << 20); break;
		case 'R': stateR |= (x << 0 ) | (y << 4 ); break;
		case 'G': stateR |= (x << 8 ) | (y << 12); break;
		case 'B': stateR |= (x << 16) | (y << 20); break;
		}
	}

	typedef unsigned short ushort;

	//Welche Zustände wir schon erreicht haben speichern wir in einem Bitarray
	//Theoretisch reichen auch 14 Bits pro Seite, aber dann kann man die
	//Einträge nicht mehr so schön mit Bitshifterei speichern. 
	ushort* states = new ushort[16*16*16*16*16*2];
	memset(states, 0, 16*16*16*16*16*2*sizeof(ushort));
	states[state>>4] |= 1 << (state & 0x0f);

	int maxCurStates = 32768;
	int* curStates = new int[maxCurStates*2];//Welche Zustände müssen wir noch weiterverfolgen
	int cCurStates = 1;
	curStates[0] = state;

	int* nextStates = curStates+maxCurStates;//Im nächsten Durchgang schauen wir uns die hier an
	int cNextStates = 0;

	//Und alles nochmal für die Rückwärtssuche
	ushort* statesR = states+16*16*16*16*16;
	statesR[stateR>>4] |= 1 << (stateR & 0x0f);

	int maxCurStatesR = 32768;
	int* curStatesR = new int[maxCurStatesR*2];
	int cCurStatesR = 1;
	curStatesR[0] = stateR;

	int* nextStatesR = curStatesR+maxCurStatesR;
	int cNextStatesR = 0;

	//Falls mal eins der Arrays zu klein wird...
	struct Helper
	{
		static void Reallocate(int*& curStates, int*& nextStates, int& maxCurStates)
		{
			int* newCurStates = new int[maxCurStates*2*2];
			memcpy(newCurStates, curStates, maxCurStates*sizeof(int));

			int* newNextStates = newCurStates+maxCurStates*2;
			memcpy(newNextStates, nextStates, maxCurStates*sizeof(int));

			delete[] std::min(curStates, nextStates);
			curStates = newCurStates;
			nextStates = newNextStates;

			maxCurStates *= 2;
		}
	};

	int moves = 1;
	for(; ; moves++)
	{
		//std::cout << cCurStates << std::endl;
		if(cCurStates < cCurStatesR)
		{
			//vorwärts suchen:
			for(int i = 0; i < cCurStates; i++)
			{
				int state = curStates[i];
				int x1 = (state >> 0) & 0x0000000f;
				int y1 = (state >> 4) & 0x0000000f;
				int x2 = (state >> 8) & 0x0000000f;
				int y2 = (state >> 12) & 0x0000000f;
				int x3 = (state >> 16) & 0x0000000f;
				int y3 = (state >> 20) & 0x0000000f;

				int x, y, newState;

				//Das hier ist nicht gerade der hübscheste Code, aber eleganter
				//hab ichs nicht hinbekommen, ohne Performanceeinbußen hinzunehmen
				//(Ich hatte da mal einen Bug, das zu debuggen war kein Spaß...)
				//f steht für "x und y flipped", v für variable, SML=smaller
#define NEW_STATE(x1, y1, x2, y2, x3, y3, x1f, y1f, x2f, y2f, x3f, y3f, xvf, yvf, delta, SML, right, mask, xShift, yShift) \
				xvf = borders[x1][y1].right; \
				yvf = y1f; \
				if(y1f == y2f && x1f SML x2f && x2f SML xvf+delta) \
					xvf = x2f-(delta); \
				if(y1f == y3f && x1f SML x3f && x3f SML xvf+delta) \
					xvf = x3f-(delta); \
				if(xvf != x1f) \
				{ \
					newState = state & mask | (x << xShift) | (y << yShift); \
					if((states[newState>>4] & (1 << (newState & 0x0f))) == 0) \
					{ \
						if(statesR[newState>>4] & (1 << (newState & 0x0f))) \
							goto End; \
						states[newState>>4] |= 1 << (newState & 0x0f); \
						if(cNextStates == maxCurStates) \
							Helper::Reallocate(curStates, nextStates, maxCurStates); \
						nextStates[cNextStates++] = newState; \
					} \
				}

				NEW_STATE(x1, y1, x2, y2, x3, y3, x1, y1, x2, y2, x3, y3, x, y,  1, <, right , 0xffffff00, 0 , 4 )
				NEW_STATE(x1, y1, x2, y2, x3, y3, x1, y1, x2, y2, x3, y3, x, y, -1, >, left  , 0xffffff00, 0 , 4 )
				NEW_STATE(x1, y1, x2, y2, x3, y3, y1, x1, y2, x2, y3, x3, y, x,  1, <, bottom, 0xffffff00, 0 , 4 )
				NEW_STATE(x1, y1, x2, y2, x3, y3, y1, x1, y2, x2, y3, x3, y, x, -1, >, top   , 0xffffff00, 0 , 4 )

				NEW_STATE(x2, y2, x1, y1, x3, y3, x2, y2, x1, y1, x3, y3, x, y,  1, <, right , 0xffff00ff, 8 , 12)
				NEW_STATE(x2, y2, x1, y1, x3, y3, x2, y2, x1, y1, x3, y3, x, y, -1, >, left  , 0xffff00ff, 8 , 12)
				NEW_STATE(x2, y2, x1, y1, x3, y3, y2, x2, y1, x1, y3, x3, y, x,  1, <, bottom, 0xffff00ff, 8 , 12)
				NEW_STATE(x2, y2, x1, y1, x3, y3, y2, x2, y1, x1, y3, x3, y, x, -1, >, top   , 0xffff00ff, 8 , 12)

				NEW_STATE(x3, y3, x1, y1, x2, y2, x3, y3, x1, y1, x2, y2, x, y,  1, <, right , 0xff00ffff, 16, 20)
				NEW_STATE(x3, y3, x1, y1, x2, y2, x3, y3, x1, y1, x2, y2, x, y, -1, >, left  , 0xff00ffff, 16, 20)
				NEW_STATE(x3, y3, x1, y1, x2, y2, y3, x3, y1, x1, y2, x2, y, x,  1, <, bottom, 0xff00ffff, 16, 20)
				NEW_STATE(x3, y3, x1, y1, x2, y2, y3, x3, y1, x1, y2, x2, y, x, -1, >, top   , 0xff00ffff, 16, 20)
#undef NEW_STATE
			}
			std::swap(curStates, nextStates);
			cCurStates = cNextStates;
			cNextStates = 0;
			if(cCurStates == 0)
			{
				moves = -1;
				break;
			}
		}
		else
		{
			//rückwärts suchen:
			for(int i = 0; i < cCurStatesR; i++)
			{
				int state = curStatesR[i];
				int x1 = (state >> 0) & 0x0000000f;
				int y1 = (state >> 4) & 0x0000000f;
				int x2 = (state >> 8) & 0x0000000f;
				int y2 = (state >> 12) & 0x0000000f;
				int x3 = (state >> 16) & 0x0000000f;
				int y3 = (state >> 20) & 0x00000000f;

				int x, y, newState;

#define NEW_STATE(x1, y1, x2, y2, x3, y3, dx, dy, mask, xShift, yShift) \
				x = x1; \
				y = y1; \
				if(level[x-(dx)][y-(dy)] == '#' || \
				   x-(dx) == x2 && y-(dy) == y2 || \
				   x-(dx) == x3 && y-(dy) == y3) \
				{ \
					x += dx; \
					y += dy; \
					while(level[x][y] != '#' && \
						  (x != x2 || y != y2) && \
						  (x != x3 || y != y3)) \
					{ \
						newState = state & mask | (x << xShift) | (y << yShift); \
						if((statesR[newState>>4] & (1 << (newState & 0x0f))) == 0) \
						{ \
							if(states[newState>>4] & (1 << (newState & 0x0f))) \
								goto End; \
							statesR[newState>>4] |= 1 << (newState & 0x0f); \
							if(cNextStatesR == maxCurStatesR) \
								Helper::Reallocate(curStatesR, nextStatesR, maxCurStatesR); \
							nextStatesR[cNextStatesR++] = newState; \
						} \
						x += dx; \
						y += dy; \
					} \
				}

				NEW_STATE(x1, y1, x2, y2, x3, y3,  1,  0, 0xffffff00, 0 , 4 )
				NEW_STATE(x1, y1, x2, y2, x3, y3, -1,  0, 0xffffff00, 0 , 4 )
				NEW_STATE(x1, y1, x2, y2, x3, y3,  0,  1, 0xffffff00, 0 , 4 )
				NEW_STATE(x1, y1, x2, y2, x3, y3,  0, -1, 0xffffff00, 0 , 4 )

				NEW_STATE(x2, y2, x1, y1, x3, y3,  1,  0, 0xffff00ff, 8 , 12)
				NEW_STATE(x2, y2, x1, y1, x3, y3, -1,  0, 0xffff00ff, 8 , 12)
				NEW_STATE(x2, y2, x1, y1, x3, y3,  0,  1, 0xffff00ff, 8 , 12)
				NEW_STATE(x2, y2, x1, y1, x3, y3,  0, -1, 0xffff00ff, 8 , 12)

				NEW_STATE(x3, y3, x1, y1, x2, y2,  1,  0, 0xff00ffff, 16, 20)
				NEW_STATE(x3, y3, x1, y1, x2, y2, -1,  0, 0xff00ffff, 16, 20)
				NEW_STATE(x3, y3, x1, y1, x2, y2,  0,  1, 0xff00ffff, 16, 20)
				NEW_STATE(x3, y3, x1, y1, x2, y2,  0, -1, 0xff00ffff, 16, 20)
#undef NEW_STATE
			}
			std::swap(curStatesR, nextStatesR);
			cCurStatesR = cNextStatesR;
			cNextStatesR = 0;
			if(cCurStatesR == 0)
			{
				moves = -1;
				break;
			}
		}
	}
End:
	delete[] states;
	delete[] std::min(curStates, nextStates);
	delete[] std::min(curStatesR, nextStatesR);
	return moves;
}
}