namespace Stef
{
int numMovesToSolve(const Level& level)
{
	// union holds position of pices only 
	union SMapPos
	{
		struct
		{
			unsigned char r;
			unsigned char g;
			unsigned char b;
			unsigned char dummy;
			
		};

		int rgb;
	};

	class CMap
	{
	private:
		// m_Map holds the structure of the map without pices.
		// included are walls, ice and target fields only.
		char m_Map[16][16];
		
		// up
		int m_up    [16][16];
		int m_down  [16][16];
		int m_left  [16][16];
		int m_right [16][16];

		// temp positions as int
		int m_rx;
		int m_ry;
		int m_gx;
		int m_gy;
		int m_bx;
		int m_by;

		// possible moves for each pice
		SMapPos m_rPos[4];
		int m_nNumrPos;
		SMapPos m_gPos[4];
		int m_nNumgPos;
		SMapPos m_bPos[4];
		int m_nNumbPos;

		// scratch pad for all possible positions
		unsigned char* m_ScratchPad;

		// target position
		SMapPos m_TargetPos;
		int m_TargetPosIndex;

	public:
		// temp counter
		int m_nCnt;

		CMap()
		{
			m_nCnt=0;
			m_ScratchPad = (unsigned char*)calloc(16777216, 1);
		};

		~CMap()
		{
			free(m_ScratchPad);
		};
		
		void Print(SMapPos& Pos)
		{
			int x, y;
			
			ConvertPosToInt(Pos);

			for(y=0; y<16; y++)
			{
				for(x=0; x<16; x++)
				{
					if(x==m_rx && y==m_ry)
						printf("r");
					else if(x==m_gx && y==m_gy)
						printf("g");
					else if(x==m_bx && y==m_by)
						printf("b");
					else
						printf("%c", m_Map[x][y]);
				}
				printf("\n");
			}
		};

		SMapPos SetMap(const char Map[16][16])
		{
			SMapPos InitPos;
			unsigned char x, y;
			int nTemp;
			
			InitPos.r=0x00;
			InitPos.g=0x00;
			InitPos.b=0x00;

			for(y=0; y<16; y++)
			{
				for(x=0; x<16; x++)
				{
					// transpose to use x and y mathematicaly correct
					m_Map[x][y] = Map[y][x];

					if(m_Map[x][y] == 'r')
					{
						m_Map[x][y] = '.';
						InitPos.r  = x<<4;
						InitPos.r += y;
					}
					else if(m_Map[x][y] == 'R')
					{
						m_TargetPos.r  = x<<4;
						m_TargetPos.r += y;
					}
					else if(m_Map[x][y] == 'g')
					{
						m_Map[x][y] = '.';
						InitPos.g  = x<<4;
						InitPos.g += y;
					}
					else if(m_Map[x][y] == 'G')
					{
						m_TargetPos.g  = x<<4;
						m_TargetPos.g += y;
					}
					else if(m_Map[x][y] == 'b')
					{
						m_Map[x][y] = '.';
						InitPos.b  = x<<4;
						InitPos.b += y;
					}
					else if(m_Map[x][y] == 'B')
					{
						m_TargetPos.b  = x<<4;
						m_TargetPos.b += y;
					}					
				}
			}

			// fill up, down, left and right table
			for(y=1; y<15; y++)
			{
				for(x=1; x<15; x++)
				{
					// up
					nTemp = y;
					while(m_Map[x][nTemp-1]!='#')
						nTemp--;
					m_up[x][y]=nTemp;

					// down
					nTemp = y;
					while(m_Map[x][nTemp+1]!='#')
						nTemp++;
					m_down[x][y]=nTemp;

					// left
					nTemp = x;
					while(m_Map[nTemp-1][y]!='#')
						nTemp--;
					m_left[x][y]=nTemp;

					// right
					nTemp = x;
					while(m_Map[nTemp+1][y]!='#')
						nTemp++;
					m_right[x][y]=nTemp;
				}
			}

			m_TargetPosIndex = m_TargetPos.rgb & 0x00FFFFFF;
			//printf("  %08X\n", TargetPosIndex);
			//printf("r:%02X\n", TargetPos.r);
			//printf("g:%02X\n", TargetPos.g);
			//printf("b:%02X\n", TargetPos.b);

			return InitPos;
		};

		bool GetMoves(SMapPos Pos, std::vector<SMapPos>& v)
		{
			int nScratchPadIndex;
			int nTemp1;
			int nTemp2;
			int nTemp3;

			m_nCnt++;
			m_nNumrPos=0;
			m_nNumgPos=0;
			m_nNumbPos=0;
			ConvertPosToInt(Pos);	

			// **************************************************************************************************************************
			// move r up
			if(m_up[m_rx][m_ry] != m_ry) 
			{
				//nTemp1 = m_ry;
				//while(m_Map[m_rx][nTemp1-1]!='#')
				//{
				//	if( m_rx==m_gx && nTemp1-1==m_gy || m_rx == m_bx && nTemp1-1==m_by)
				//		break;
				//	nTemp1--;
				//}
				nTemp1 = m_up[m_rx][m_ry];
				if(m_rx == m_gx && m_rx != m_bx)
				{
					if(m_ry>m_gy && nTemp1<=m_gy)
						nTemp1=m_gy+1;
				}
				if(m_rx != m_gx && m_rx == m_bx)
				{
					if(m_ry>m_by && nTemp1<=m_by)
						nTemp1=m_by+1;					
				}
				if(m_rx == m_gx && m_rx == m_bx) // both on same y level
				{
					if(m_gy>m_by)
					{
						if(m_ry>m_gy && nTemp1<=m_gy)
							nTemp1=m_gy+1;
						else if(m_ry>m_by && nTemp1<=m_by)
							nTemp1=m_by+1;
					}
					else
					{
						if(m_ry>m_by && nTemp1<=m_by)
							nTemp1=m_by+1;
						else if(m_ry>m_gy && nTemp1<=m_gy)
							nTemp1=m_gy+1;
					}
				}	


				if(m_ry != nTemp1)
				{
					nTemp2 = m_ry;
					m_ry = nTemp1;
					ConvertIntToPos(m_rPos[m_nNumrPos++]);
					m_ry = nTemp2;
				}
			}

			// move r down
			if(m_down[m_rx][m_ry] != m_ry) 
			{
				//nTemp1 = m_ry;
				//while(m_Map[m_rx][nTemp1+1]!='#')
				//{
				//	if( m_rx==m_gx && nTemp1+1==m_gy || m_rx == m_bx && nTemp1+1==m_by)
				//		break;
				//	nTemp1++;
				//}
				nTemp1 = m_down[m_rx][m_ry];
				if(m_rx == m_gx && m_rx != m_bx)
				{
					if(m_ry<m_gy && nTemp1>=m_gy)
						nTemp1=m_gy-1;
				}
				if(m_rx != m_gx && m_rx == m_bx)
				{
					if(m_ry<m_by && nTemp1>=m_by)
						nTemp1=m_by-1;					
				}
				if(m_rx == m_gx && m_rx == m_bx) // both on same y level
				{
					if(m_gy<m_by)
					{
						if(m_ry<m_gy && nTemp1>=m_gy)
							nTemp1=m_gy-1;
						else if(m_ry<m_by && nTemp1>=m_by)
							nTemp1=m_by-1;
					}
					else
					{
						if(m_ry<m_by && nTemp1>=m_by)
							nTemp1=m_by-1;
						else if(m_ry<m_gy && nTemp1>=m_gy)
							nTemp1=m_gy-1;
					}
				}	

				if(m_ry != nTemp1)
				{
					nTemp2 = m_ry;
					m_ry = nTemp1;
					ConvertIntToPos(m_rPos[m_nNumrPos++]);
					m_ry = nTemp2;
				}
			}

			// move r left
			if(m_left[m_rx][m_ry] != m_rx) 
			{
				//nTemp1 = m_rx;
				//while(m_Map[nTemp1-1][m_ry]!='#')
				//{
				//	if( nTemp1-1==m_gx && m_ry==m_gy || nTemp1-1==m_bx && m_ry==m_by)
				//		break;
				//	nTemp1--;
				//}
				nTemp1 = m_left[m_rx][m_ry];
				if(m_ry == m_gy && m_ry != m_by)
				{
					if(m_rx>m_gx && nTemp1<=m_gx)
						nTemp1=m_gx+1;
				}
				if(m_ry != m_gy && m_ry == m_by)
				{
					if(m_rx>m_bx && nTemp1<=m_bx)
						nTemp1=m_bx+1;					
				}
				if(m_ry == m_gy && m_ry == m_by) // both on same y level
				{
					if(m_gx>m_bx)
					{
						if(m_rx>m_gx && nTemp1<=m_gx)
							nTemp1=m_gx+1;
						else if(m_rx>m_bx && nTemp1<=m_bx)
							nTemp1=m_bx+1;
					}
					else
					{
						if(m_rx>m_bx && nTemp1<=m_bx)
							nTemp1=m_bx+1;
						else if(m_rx>m_gx && nTemp1<=m_gx)
							nTemp1=m_gx+1;
					}
				}	

				if(m_rx != nTemp1)
				{
					nTemp2 = m_rx;
					m_rx = nTemp1;
					ConvertIntToPos(m_rPos[m_nNumrPos++]);
					m_rx = nTemp2;
				}
			}

			// move r right
			if(m_right[m_rx][m_ry] != m_rx) 
			{
				//nTemp1 = m_rx;
				//while(m_Map[nTemp1+1][m_ry]!='#')
				//{
				//	if( nTemp1+1==m_gx && m_ry==m_gy || nTemp1+1==m_bx && m_ry==m_by)
				//		break;
				//	nTemp1++;
				//}
				nTemp1 = m_right[m_rx][m_ry];
				if(m_ry == m_gy && m_ry != m_by)
				{
					if(m_rx<m_gx && nTemp1>=m_gx)
						nTemp1=m_gx-1;
				}
				if(m_ry != m_gy && m_ry == m_by)
				{
					if(m_rx<m_bx && nTemp1>=m_bx)
						nTemp1=m_bx-1;					
				}
				if(m_ry == m_gy && m_ry == m_by) // both on same y level
				{
					if(m_gx<m_bx)
					{
						if(m_rx<m_gx && nTemp1>=m_gx)
							nTemp1=m_gx-1;
						else if(m_rx<m_bx && nTemp1>=m_bx)
							nTemp1=m_bx-1;
					}
					else
					{
						if(m_rx<m_bx && nTemp1>=m_bx)
							nTemp1=m_bx-1;
						else if(m_rx<m_gx && nTemp1>=m_gx)
							nTemp1=m_gx-1;
					}
				}	

				if(m_rx != nTemp1)
				{
					nTemp2 = m_rx;
					m_rx = nTemp1;
					ConvertIntToPos(m_rPos[m_nNumrPos++]);
					m_rx = nTemp2;
				}				
			}

			// **************************************************************************************************************************
			// move g up
			if(m_up[m_gx][m_gy] != m_gy) 
			{
				//nTemp1 = m_gy;
				//while(m_Map[m_gx][nTemp1-1]!='#')
				//{
				//	if( m_gx==m_rx && nTemp1-1==m_ry || m_gx==m_bx && nTemp1-1==m_by)
				//		break;
				//	nTemp1--;
				//}
				nTemp1 = m_up[m_gx][m_gy];
				if(m_gx == m_rx && m_gx != m_bx)
				{
					if(m_gy>m_ry && nTemp1<=m_ry)
						nTemp1=m_ry+1;
				}
				if(m_gx != m_rx && m_gx == m_bx)
				{
					if(m_gy>m_by && nTemp1<=m_by)
						nTemp1=m_by+1;					
				}
				if(m_gx == m_rx && m_gx == m_bx) // both on same y level
				{
					if(m_ry>m_by)
					{
						if(m_gy>m_ry && nTemp1<=m_ry)
							nTemp1=m_ry+1;
						else if(m_gy>m_by && nTemp1<=m_by)
							nTemp1=m_by+1;
					}
					else
					{
						if(m_gy>m_by && nTemp1<=m_by)
							nTemp1=m_by+1;
						else if(m_gy>m_ry && nTemp1<=m_ry)
							nTemp1=m_ry+1;
					}
				}	

				if(m_gy != nTemp1)
				{
					nTemp2 = m_gy;
					m_gy = nTemp1;
					ConvertIntToPos(m_gPos[m_nNumgPos++]);
					m_gy = nTemp2;
				}
			}

			// move g down
			if(m_down[m_gx][m_gy] != m_gy) 
			{
				//nTemp1 = m_gy;
				//while(m_Map[m_gx][nTemp1+1]!='#')
				//{
				//	if( m_gx==m_rx && nTemp1+1==m_ry || m_gx==m_bx && nTemp1+1==m_by)
				//		break;
				//	nTemp1++;
				//}
				nTemp1 = m_down[m_gx][m_gy];
				if(m_gx == m_rx && m_gx != m_bx)
				{
					if(m_gy<m_ry && nTemp1>=m_ry)
						nTemp1=m_ry-1;
				}
				if(m_gx != m_rx && m_gx == m_bx)
				{
					if(m_gy<m_by && nTemp1>=m_by)
						nTemp1=m_by-1;					
				}
				if(m_gx == m_rx && m_gx == m_bx) // both on same y level
				{
					if(m_ry<m_by)
					{
						if(m_gy<m_ry && nTemp1>=m_ry)
							nTemp1=m_ry-1;
						else if(m_gy<m_by && nTemp1>=m_by)
							nTemp1=m_by-1;
					}
					else
					{
						if(m_gy<m_by && nTemp1>=m_by)
							nTemp1=m_by-1;
						else if(m_gy<m_ry && nTemp1>=m_ry)
							nTemp1=m_ry-1;
					}
				}	

				if(m_gy != nTemp1)
				{
					nTemp2 = m_gy;
					m_gy = nTemp1;
					ConvertIntToPos(m_gPos[m_nNumgPos++]);
					m_gy = nTemp2;
				}
			}

			// move g left
			if(m_left[m_gx][m_gy] != m_gx) 
			{
				//nTemp1 = m_gx;
				//while(m_Map[nTemp1-1][m_gy]!='#')
				//{
				//	if( nTemp1-1==m_rx && m_gy==m_ry || nTemp1-1==m_bx && m_gy==m_by)
				//		break;
				//	nTemp1--;
				//}
				nTemp1 = m_left[m_gx][m_gy];
				if(m_gy == m_ry && m_gy != m_by)
				{
					if(m_gx>m_rx && nTemp1<=m_rx)
						nTemp1=m_rx+1;
				}
				if(m_gy != m_ry && m_gy == m_by)
				{
					if(m_gx>m_bx && nTemp1<=m_bx)
						nTemp1=m_bx+1;					
				}
				if(m_gy == m_ry && m_gy == m_by) // both on same y level
				{
					if(m_rx>m_bx)
					{
						if(m_gx>m_rx && nTemp1<=m_rx)
							nTemp1=m_rx+1;
						else if(m_gx>m_bx && nTemp1<=m_bx)
							nTemp1=m_bx+1;
					}
					else
					{
						if(m_gx>m_bx && nTemp1<=m_bx)
							nTemp1=m_bx+1;
						else if(m_gx>m_rx && nTemp1<=m_rx)
							nTemp1=m_rx+1;
					}
				}	

				if(m_gx != nTemp1)
				{
					nTemp2 = m_gx;
					m_gx = nTemp1;
					ConvertIntToPos(m_gPos[m_nNumgPos++]);
					m_gx = nTemp2;
				}
			}

			// move g right
			if(m_right[m_gx][m_gy] != m_gx) 
			{
				//nTemp1 = m_gx;
				//while(m_Map[nTemp1+1][m_gy]!='#')
				//{
				//	if( nTemp1+1==m_rx && m_gy==m_ry || nTemp1+1==m_bx && m_gy==m_by)
				//		break;
				//	nTemp1++;
				//}
				nTemp1 = m_right[m_gx][m_gy];
				if(m_gy == m_ry && m_gy != m_by)
				{
					if(m_gx<m_rx && nTemp1>=m_rx)
						nTemp1=m_rx-1;
				}
				if(m_gy != m_ry && m_gy == m_by)
				{
					if(m_gx<m_bx && nTemp1>=m_bx)
						nTemp1=m_bx-1;					
				}
				if(m_gy == m_ry && m_gy == m_by) // both on same y level
				{
					if(m_rx<m_bx)
					{
						if(m_gx<m_rx && nTemp1>=m_rx)
							nTemp1=m_rx-1;
						else if(m_gx<m_bx && nTemp1>=m_bx)
							nTemp1=m_bx-1;
					}
					else
					{
						if(m_gx<m_bx && nTemp1>=m_bx)
							nTemp1=m_bx-1;
						else if(m_gx<m_rx && nTemp1>=m_rx)
							nTemp1=m_rx-1;
					}
				}	

				if(m_gx != nTemp1)
				{
					nTemp2 = m_gx;
					m_gx = nTemp1;
					ConvertIntToPos(m_gPos[m_nNumgPos++]);
					m_gx = nTemp2;
				}
			}

			// **************************************************************************************************************************
			// move b up
			if(m_up[m_bx][m_by] != m_by) 
			{
				//nTemp1 = m_by;
				//while(m_Map[m_bx][nTemp1-1]!='#')
				//{
				//	if( m_bx==m_rx && nTemp1-1==m_ry || m_bx==m_gx && nTemp1-1==m_gy)
				//		break;
				//	nTemp1--;
				//}
				nTemp1 = m_up[m_bx][m_by];
				if(m_bx == m_rx && m_bx != m_gx)
				{
					if(m_by>m_ry && nTemp1<=m_ry)
						nTemp1=m_ry+1;
				}
				if(m_bx != m_rx && m_bx == m_gx)
				{
					if(m_by>m_gy && nTemp1<=m_gy)
						nTemp1=m_gy+1;					
				}
				if(m_bx == m_rx && m_bx == m_gx) // both on same y level
				{
					if(m_ry>m_gy)
					{
						if(m_by>m_ry && nTemp1<=m_ry)
							nTemp1=m_ry+1;
						else if(m_by>m_gy && nTemp1<=m_gy)
							nTemp1=m_gy+1;
					}
					else
					{
						if(m_by>m_gy && nTemp1<=m_gy)
							nTemp1=m_gy+1;
						else if(m_by>m_ry && nTemp1<=m_ry)
							nTemp1=m_ry+1;
					}
				}	

				if(m_by != nTemp1)
				{
					nTemp2 = m_by;
					m_by = nTemp1;
					ConvertIntToPos(m_bPos[m_nNumbPos++]);
					m_by = nTemp2;
				}
			}

			// move b down
			if(m_down[m_bx][m_by] != m_by) 
			{
				//nTemp1 = m_by;
				//while(m_Map[m_bx][nTemp1+1]!='#')
				//{
				//	if( m_bx==m_rx && nTemp1+1==m_ry || m_bx==m_gx && nTemp1+1==m_gy)
				//		break;
				//	nTemp1++;
				//}
				nTemp1 = m_down[m_bx][m_by];
				if(m_bx == m_rx && m_bx != m_gx)
				{
					if(m_by<m_ry && nTemp1>=m_ry)
						nTemp1=m_ry-1;
				}
				if(m_bx != m_rx && m_bx == m_gx)
				{
					if(m_by<m_gy && nTemp1>=m_gy)
						nTemp1=m_gy-1;					
				}
				if(m_bx == m_rx && m_bx == m_gx) // both on same y level
				{
					if(m_ry<m_gy)
					{
						if(m_by<m_ry && nTemp1>=m_ry)
							nTemp1=m_ry-1;
						else if(m_by<m_gy && nTemp1>=m_gy)
							nTemp1=m_gy-1;
					}
					else
					{
						if(m_by<m_gy && nTemp1>=m_gy)
							nTemp1=m_gy-1;
						else if(m_by<m_ry && nTemp1>=m_ry)
							nTemp1=m_ry-1;
					}
				}	

				if(m_by != nTemp1)
				{
					nTemp2 = m_by;
					m_by = nTemp1;
					ConvertIntToPos(m_bPos[m_nNumbPos++]);
					m_by = nTemp2;
				}
			}

			// move b left
			if(m_left[m_bx][m_by] != m_bx) 
			{
				//nTemp1 = m_bx;
				//while(m_Map[nTemp1-1][m_by]!='#')
				//{
				//	if( nTemp1-1==m_rx && m_by==m_ry || nTemp1-1==m_gx && m_by==m_gy)
				//		break;
				//	nTemp1--;
				//}
				nTemp1 = m_left[m_bx][m_by];
				if(m_by == m_ry && m_by != m_gy)
				{
					if(m_bx>m_rx && nTemp1<=m_rx)
						nTemp1=m_rx+1;
				}
				if(m_by != m_ry && m_by == m_gy)
				{
					if(m_bx>m_gx && nTemp1<=m_gx)
						nTemp1=m_gx+1;					
				}
				if(m_by == m_ry && m_by == m_gy) // both on same y level
				{
					if(m_rx>m_gx)
					{
						if(m_bx>m_rx && nTemp1<=m_rx)
							nTemp1=m_rx+1;
						else if(m_bx>m_gx && nTemp1<=m_gx)
							nTemp1=m_gx+1;
					}
					else
					{
						if(m_bx>m_gx && nTemp1<=m_gx)
							nTemp1=m_gx+1;
						else if(m_bx>m_rx && nTemp1<=m_rx)
							nTemp1=m_rx+1;
					}
				}	

				if(m_bx != nTemp1)
				{
					nTemp2 = m_bx;
					m_bx = nTemp1;
					ConvertIntToPos(m_bPos[m_nNumbPos++]);
					m_bx = nTemp2;
				}
			}

			// move b right
			if(m_right[m_bx][m_by] != m_bx) 
			{
				//nTemp1 = m_bx;
				//while(m_Map[nTemp1+1][m_by]!='#')
				//{
				//	if( nTemp1+1==m_rx && m_by==m_ry || nTemp1+1==m_gx && m_by==m_gy)
				//		break;
				//	nTemp1++;
				//}
				nTemp1 = m_right[m_bx][m_by];
				if(m_by == m_ry && m_by != m_gy)
				{
					if(m_bx<m_rx && nTemp1>=m_rx)
						nTemp1=m_rx-1;
				}
				if(m_by != m_ry && m_by == m_gy)
				{
					if(m_bx<m_gx && nTemp1>=m_gx)
						nTemp1=m_gx-1;					
				}
				if(m_by == m_ry && m_by == m_gy) // both on same y level
				{
					if(m_rx<m_gx)
					{
						if(m_bx<m_rx && nTemp1>=m_rx)
							nTemp1=m_rx-1;
						else if(m_bx<m_gx && nTemp1>=m_gx)
							nTemp1=m_gx-1;
					}
					else
					{
						if(m_bx<m_gx && nTemp1>=m_gx)
							nTemp1=m_gx-1;
						else if(m_bx<m_rx && nTemp1>=m_rx)
							nTemp1=m_rx-1;
					}
				}	

				if(m_bx != nTemp1)
				{
					nTemp2 = m_bx;
					m_bx = nTemp1;
					ConvertIntToPos(m_bPos[m_nNumbPos++]);
					m_bx = nTemp2;
				}
			}
			
			// add all possible combinations to v
			for(nTemp1=0; nTemp1<m_nNumrPos; nTemp1++) // nTemp1 == rot
			{
				nScratchPadIndex = m_rPos[nTemp1].rgb & 0x00FFFFFF;
				
				if(m_TargetPosIndex == nScratchPadIndex)
					return true;

				if(m_ScratchPad[nScratchPadIndex] == 0x00)
				{
					m_ScratchPad[nScratchPadIndex] = 0x01;
					v.push_back(m_rPos[nTemp1]);
				}
			}

			for(nTemp2=0; nTemp2<m_nNumgPos; nTemp2++) // nTemp2 == gruen
			{
				nScratchPadIndex = m_gPos[nTemp2].rgb & 0x00FFFFFF;

				if(m_TargetPosIndex == nScratchPadIndex)
					return true;

				if(m_ScratchPad[nScratchPadIndex] == 0x00)
				{
					m_ScratchPad[nScratchPadIndex] = 0x01;
					v.push_back(m_gPos[nTemp2]);
				}
			}

			for(nTemp3=0; nTemp3<m_nNumbPos; nTemp3++) // nTemp3 == blau
			{
				nScratchPadIndex = m_bPos[nTemp3].rgb & 0x00FFFFFF;

				if(m_TargetPosIndex == nScratchPadIndex)
					return true;

				if(m_ScratchPad[nScratchPadIndex] == 0x00)
				{
					m_ScratchPad[nScratchPadIndex] = 0x01;
					v.push_back(m_bPos[nTemp3]);
				}
			}

			return false;
		};

	private:
		void ConvertPosToInt(SMapPos& Pos)
		{
			// convert positions to int
			m_rx = (Pos.r >> 4) & 0x0000000F;
			m_ry = Pos.r & 0x0000000F;
			m_gx = (Pos.g >> 4) & 0x0000000F;
			m_gy = Pos.g & 0x0000000F;
			m_bx = (Pos.b >> 4) & 0x0000000F;
			m_by = Pos.b & 0x0000000F;
		}

		void ConvertIntToPos(SMapPos& Pos)
		{
			// convert int to position
			Pos.r  = m_rx<<4;
			Pos.r += m_ry;
			Pos.g  = m_gx<<4;
			Pos.g += m_gy;
			Pos.b  = m_bx<<4;
			Pos.b += m_by;
		}
	};


	int i;
	bool bRet;
	std::vector<SMapPos> Move[257];
	std::vector<SMapPos>::iterator iterator;
	std::vector<SMapPos>::iterator iterator_end;
	CMap Map;

	// copy initial position to Move[0]
	Move[0].push_back( Map.SetMap(level) );

	try
	{
		for(i=0; i<=255; i++)
		{
			Move[i].reserve(i*100+1);
			iterator = Move[i].begin();
			iterator_end = Move[i].end();
			while(iterator != iterator_end)
			{
				try
				{
					bRet = Map.GetMoves(*iterator, Move[i+1]);
					if(bRet == true)
						return i+1;
				}
				catch(std::bad_alloc)
				{
					printf("Out of memory\n");
					return -1;
				}

				iterator++;
			}
		}
	}
	catch(...)
	{
		printf("Unknown exception\n");
	}

	return -1;
}
}